#include "chatoperator.h"

#include "global/kaglobal.h"

#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonValue>
#include <QThread>
#include <QTime>
#include <ukui-log4qt.h>

bool is_reply_end = true;

class ChatOperator::Private
{
public:
    void *session = nullptr;
};

QString getErrorMsg(NlpResult errorCode)
{
    QString errorMsg;
    switch(errorCode)
    {
    case NLP_SUCCESS:
        break;
    case NLP_NET_ERROR:
        qDebug() << "NLP 网络设置错误。";
        errorMsg = "未检测到网络链接。请确认网络设置后再试。";
        break;
    case NLP_SESSION_ERROR:
        qDebug() << "session错误。";
        errorMsg = "Session错误，请重启服务再试。";
        break;
    case NLP_SERVER_ERROR:
        qDebug() << "服务错误或者不可用。";
        errorMsg = "服务错误或者不可用。请切换网络再试。";
        break;
    case NLP_UNAUTHORIZED:
        qDebug() << "权限限制，鉴权失败，无权访问。";
        errorMsg = "鉴权失败，请重新配置KEY。";
        break;
    case NLP_REQUEST_TOO_MANY:
        qDebug() << "请求频率过高。";
        errorMsg = "请求频率过高，请等一会再试。";
        break;
    case NLP_REQUEST_FAILED:
        qDebug() << "请求失败。";
        errorMsg = "请求失败，请重新发送。";
        break;
    case NLP_INPUT_TEXT_LENGTH_INVAILD:
        qDebug() << "输入文本长度超限。";
        errorMsg = "输入文本长度超限，请重新编辑文本再发送。";
        break;
    case NLP_PARAM_ERROR:
        qDebug() << "参数错误。";
        errorMsg = "鉴权失败，请重新配置KEY。";
        break;
    default:
        break;
    }
    return errorMsg;
}

/**
 * @brief chatCallback
 *          聊天结果返回回调函数，数据流式返回
 * @param text_data
 * @param usr_data
 */
static void chatCallback(NlpTextData text_data, void *usr_data)
{
    ChatOperator *chatOperator = (ChatOperator *)usr_data;
    int errorCode = text_data.error_code;
    char *result = (char *)text_data.text;
    QJsonParseError jsonError;
    QJsonDocument doc = QJsonDocument::fromJson(result, &jsonError);
    if(jsonError.error == QJsonParseError::NoError)
    {
        QJsonObject obj = doc.object();
        chatOperator->is_end = obj.value("is_end").toBool();
        chatOperator->result_msg = obj.value("result").toString();
        chatOperator->result_index = obj.value("sentence_id").toInt();
    }
    else
    {
        qDebug() << "JSON parsing error: " << jsonError.errorString();
    }
    is_reply_end = chatOperator->is_end;
    qDebug() << "is_reply_end" << is_reply_end;
    isNlpInUse = !chatOperator->is_end;

    switch(chatOperator->target)
    {
    case TartgetType::RECVMSG:
        if(errorCode != 0)
        {
            QString errorMsg = getErrorMsg((NlpResult)errorCode);
            chatOperator->notifyUI(errorMsg, chatOperator->is_end, chatOperator->result_index, errorCode);
            break;
        }
        chatOperator->notifyUI(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index, errorCode);
        //语音模式下记录当前回答的所有内容并在回答结束后进行朗读
        if(chatOperator->soundFlag)
        {
            if(chatOperator->is_end)
            {
                //回答结束后进行将所有的回答发送到朗读接口进行朗
                emit chatOperator->sigSoundMsg(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index);
                chatOperator->soundFlag = false; //设置语音回答标志位
            }
        }
        break;
    case TartgetType::SUMMARY:
        emit chatOperator->sigGenerateSummary(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index, errorCode);
        break;
    case TartgetType::CALENDAR:
        emit chatOperator->sigAddToSchedule(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index, errorCode);
        break;
    }
}

ChatOperator::ChatOperator(QObject *parent)
    : QObject { parent }
    , d(std::make_shared<ChatOperator::Private>())
{
}

ChatOperator::~ChatOperator()
{
    if(d->session)
        nlp_destroy_session(d->session);
}

bool ChatOperator::sendMessage(const QString &message)
{
    if(!initSession())
    {
        return false;
    }
    QString send_message = message;
    QThread::create([this, send_message]() {
        NlpResult ret = nlp_text_chat_async(d->session, send_message.toStdString().c_str());
        if(ret == NLP_SUCCESS)
        {
            qDebug() << "nlp_text_chat_async successful";
        }
        else
        {
            QString error;
            qDebug() << "nlp_text_chat_async error:" << ret;
            error = getErrorMsg(ret);
            emit reply(error, true, 0, ret);
        }
    })->start();

    return true;
}

void ChatOperator::notifyUI(const QString &message, bool is_end, int index, int errorCode)
{
    if(is_end)
    {
        emit sigSessionCompleted();
    }

    emit reply(message, is_end, index, errorCode);
}

void ChatOperator::setActor(int actorId)
{
    qDebug() << "actorId" << actorId;
    nlp_set_session_actor(d->session, actorId);
}

bool ChatOperator::initModelVendor()
{
    QJsonObject obj;
    obj.insert("apiKey", "0bLvOdqlo1Gzu2jbpf9imTsT");
    obj.insert("secretKey", "H0ThDOxNvTKEbNgyeHyKdw35R6dw3AcE");

    return set_remote_model_config(BAIDU, NLP, QJsonDocument(obj).toJson().toStdString().c_str());
}

bool ChatOperator::initSession()
{
    if(d->session)
        return true;

    QString error;
    void *session = nullptr;
    // 创建session
    NlpResult ret = nlp_create_session(&session);
    if(ret == NLP_SUCCESS)
    {
        qDebug() << "nlp_create_session successful";
    }
    else
    {
        qDebug() << "nlp_create_session error:" << ret;
        error = getErrorMsg(ret);
        emit reply(error, true, 0, ret);
        isNlpInUse = false;
        return false;
    }

    // 初始化session
    ret = nlp_init_session(session);
    if(ret == NLP_SUCCESS)
    {
        qDebug() << "nlp_init_session successful";
    }
    else
    {
        qDebug() << "nlp_init_session error:" << ret;
        error = getErrorMsg(ret);
        emit reply(error, true, 0, ret);
        isNlpInUse = false;
        return false;
    }
    if(!ret)
    {
        d->session = session;
        nlp_set_result_callback(d->session, chatCallback, this);
    }

    return true;
}

void ChatOperator::closeSession()
{
    if(d->session)
        nlp_destroy_session(d->session);

    d->session = nullptr;
}

void *ChatOperator::getSession()
{
    return d->session;
}

void ChatOperator::clearSession()
{
    if(d->session)
    {
        nlp_clear_context(d->session);
    }

    return;
}

void ChatOperator::stopChat()
{
    if(d->session)
    {
        if(nlp_stop_chat(d->session) != NLP_SUCCESS)
            qDebug() << "nlp stop chat error";
        else
            isNlpInUse = false;
    }
    return;
}
