#include "meetingrecorder.h"

#include <QJsonDocument>
#include <QJsonObject>
#include <QThread>
#include <kylin-ai/ai-base/speech.h>
#include <ukui-log4qt.h>

struct SpeechItme
{
    QString name;
    QString start, end;
    QString content;
};
static QJsonObject jsonObject;

QString MeetingRecorder::getErrorMsg(SpeechResult errorCode)
{
    QString errorMsg;
    switch(errorCode)
    {
    case SPEECH_SESSION_ERROR:
        errorMsg = "会话创建错误，请检查模型配置是否正确。";
        break;
    case SPEECH_NET_ERROR:
        errorMsg = "网络出现问题，请调整网络后重试。";
        break;
    case SPEECH_SERVER_ERROR:
        errorMsg = "服务器出现问题，请调整网络后重试。";
        break;
    case SPEECH_UNAUTHORIZED:
        errorMsg = "鉴权失败，请确认API KEY,SECRET KEY是否正确。";
        break;
    case SPEECH_REQUEST_TOO_MANY:
        errorMsg = "请求过于频繁，请稍等后重试。";
        break;
    case SPEECH_REQUEST_FAILED:
        errorMsg = "请求失败。";
        break;
    case SPEECH_TEXT_LENGTH_INVAILD:
        errorMsg = "文本超限，请精简文本后重试。";
        break;
    case SPEECH_ADIO_SIZE_INVAILD:
        errorMsg = "音频大小超限，请重新尝试";
        break;
    case SPEECH_PARAM_ERROR:
        errorMsg = "参数错误，请修改后重试。";
        break;
    default:
        errorMsg = "未知错误！";
        break;
    };
    return errorMsg;
}

/**
 * @brief runtimeSpeechCallback
 *          聊天结果返回回调函数，数据流式返回
 * @param result
 * @param usr_data
 */
static void finishedCallback(RecognitionResult result, void *user_data)
{
    if(user_data == nullptr)
    {
        qDebug() << "Invalid callback user data pointer.";
    }
    int errorCode = result.error_code;
    MeetingRecorder *meetingRecorder = (MeetingRecorder *)user_data;
    if(errorCode != 0)
    {
        //百度模型，五秒钟未收到音频会自动断开连接，导致会议助手暂停记录后无法继续使用
        qDebug() << "meeting speech callback error, errorcode :" << errorCode;
        //        QString str = meetingRecorder->getErrorMsg((SpeechResult)errorCode);
        //        emit meetingRecorder->sigErrorMsg(str);
        return;
    }
    emit meetingRecorder->sigAppendSpeech(result.speaker_id, result.text_data);
}

/**
 * @brief runtimeSpeechCallback
 *          聊天结果返回回调函数，数据流式返回
 * @param result
 * @param usr_data
 */
static void MeetingCallback(RecognitionResult result, void *usr_data)
{
    if(usr_data == nullptr)
    {
        qDebug() << "Invalid callback user data pointer.";
    }
    MeetingRecorder *meetingRecorder = (MeetingRecorder *)usr_data;
    emit meetingRecorder->sigShowSpeech(result.text_data);
}

MeetingRecorder::MeetingRecorder(QObject *parent)
    : QObject(parent)
{
    //设置录音格式
    m_audioFormat.setByteOrder(QAudioFormat::LittleEndian);
    m_audioFormat.setChannelCount(1);
    m_audioFormat.setCodec("audio/pcm");
    m_audioFormat.setSampleRate(16000);
    m_audioFormat.setSampleSize(16);
    m_audioFormat.setSampleType(QAudioFormat::SignedInt);

    // 判断设备，查看是否存在
    QAudioDeviceInfo devInfo = QAudioDeviceInfo::defaultInputDevice();
    if(devInfo.isNull())
    {
        qWarning() << "No recording device was found!";
    }
    // 不支持格式，使用最接近格式
    if(!devInfo.isFormatSupported(m_audioFormat))
    { // 当前使用设备是否支持
        qWarning() << "Automatic conversion is made to the nearest format!";
        m_audioFormat = devInfo.nearestFormat(m_audioFormat); // 转换为最接近格式
    }

    m_audioInput = new QAudioInput(devInfo, m_audioFormat, this);

    m_audioInput->setNotifyInterval(10 * 1000);

    jsonObject["format"] = "pcm";
    jsonObject["rate"] = 16000;
    jsonObject["channel"] = 1;
}

MeetingRecorder::~MeetingRecorder()
{
    if(m_audioInput != nullptr)
    {
        m_audioInput->stop();
        delete m_audioInput;
    }
    closeSession();
}

bool MeetingRecorder::startMeetingRecord()
{

    if(!initSession())
    {
        qDebug() << "startRecord  meeting error!";
        return false;
    }
    qDebug() << "startRecord  meeting";
    m_audioRecInfo.open(QIODevice::ReadWrite | QIODevice::Truncate);
    m_audioInput->start(&m_audioRecInfo);
    const char *params = QJsonDocument(jsonObject).toJson(QJsonDocument::Indented);
    speech_start_continuous_recognition(this->session, params);
    return true;
}

void MeetingRecorder::pauseMeetingRecord()
{
    qDebug() << "pause meeting record";
    m_audioInput->stop();
    m_audioRecInfo.close();
    if(this->session)
        speech_stop_continuous_recognition(this->session);
}

void MeetingRecorder::continueMeetingRecord()
{
    qDebug() << "continue meeting record";
    m_audioRecInfo.open(QIODevice::ReadWrite | QIODevice::Truncate);
    m_audioInput->start(&m_audioRecInfo);
    if(this->session)
    {
        const char *params = QJsonDocument(jsonObject).toJson(QJsonDocument::Indented);
        speech_start_continuous_recognition(this->session, params);
    }
}

void MeetingRecorder::stopMeetingRecord()
{
    m_audioInput->stop();
    m_audioRecInfo.close();
    speech_stop_continuous_recognition(this->session);
}

bool MeetingRecorder::initSession()
{
    if(this->session != nullptr)
        return true;
    QString error;
    void *session = nullptr;
    SpeechResult ret = speech_create_session(&session);
    if(ret == SPEECH_SUCCESS)
    {
        KyInfo() << "meeting speech_create_session successful";
    }
    else
    {
        KyInfo() << "speech_create_session error:" << ret;
        error = getErrorMsg(ret);
        emit sigErrorMsg(error);
        //        emit reply(error, true, 0, ret);
        //        emit sigCancelSendVoice();
        return false;
    }
    ret = speech_init_session(session);
    if(ret == SPEECH_SUCCESS)
    {
        KyInfo() << "meeting speech_init_session successful";
    }
    else
    {
        KyInfo() << "speech_init_session error:" << ret;
        error = getErrorMsg(ret);
        emit sigErrorMsg(error);
        //        emit reply(error, true, 0, ret);
        //        emit sigCancelSendVoice();
        return false;
    }

    if(session != nullptr)
    {
        this->session = session;
        //设置语音识别文字回调
        speech_set_recognizing_callback(this->session, MeetingCallback, this);
        //翻译一段语音识别，触发回调函数
        speech_set_recognized_callback(this->session, finishedCallback, this);
        m_audioRecInfo.setAudioSession(session);
    }

    return true;
}

void MeetingRecorder::closeSession()
{
    if(this->session != nullptr)
        speech_destroy_session(this->session);
    this->session = nullptr;
}
