#include "recorder.h"
#include "global/kaglobal.h"

#include <QIODevice>
#include <QThread>
#include <ukui-log4qt.h>

QString getErrorMsg(SpeechResult errorCode)
{
    QString errorMsg;
    switch(errorCode)
    {
    case SPEECH_SESSION_ERROR:
        errorMsg = "Session错误，请重启服务再试。";
        break;
    case SPEECH_NET_ERROR:
        errorMsg = "未检测到网络链接。请确认网络设置后再试。";
        break;
    case SPEECH_SERVER_ERROR:
        errorMsg = "服务错误或者不可用。请切换网络再试。";
        break;
    case SPEECH_UNAUTHORIZED:
        errorMsg = "权限限制，鉴权失败，无权访问。";
        break;
    case SPEECH_REQUEST_TOO_MANY:
        errorMsg = "请求频率过高，请等一会再试。";
        break;
    case SPEECH_REQUEST_FAILED:
        errorMsg = "请求失败。";
        break;
    case SPEECH_TEXT_LENGTH_INVAILD:
        errorMsg = "输入音频长度超限，请重新发送。";
        break;
    case SPEECH_ADIO_SIZE_INVAILD:
        errorMsg = "音频大小超限，请重新尝试";
        break;
    case SPEECH_PARAM_ERROR:
        errorMsg = "鉴权失败，请重新配置KEY。";
        break;
    default:
        errorMsg = "未知错误！";
        break;
    };
    return errorMsg;
}

/**
 * @brief chatCallback
 *          聊天结果返回回调函数，数据流式返回
 * @param result
 * @param usr_data
 */
static void speechCallback(RecognitionResult result, void *usr_data)
{
    if(usr_data == nullptr)
    {
        qDebug() << "Invalid callback user data pointer.";
    }
    int errorCode = result.error_code;
    qDebug() << "ddddddddddd" << errorCode;
    char *result_text = (char *)result.text_data;
    qDebug() << "ddddddddddd" << result_text;
    Recorder *recorder = (Recorder *)usr_data;
    if(errorCode != 0)
    {
        QString errorMsg = getErrorMsg((SpeechResult)errorCode);
        emit recorder->reply(errorMsg, true, 0, errorCode);
        emit recorder->sigCancelSendVoice();
    }
    else
    {
        emit recorder->sigSendMsg(result.text_data);
    }
}
Recorder::Recorder(QObject *parent)
    : QObject(parent)
{
    //设置录音格式
    m_audioFormat.setByteOrder(QAudioFormat::LittleEndian);
    m_audioFormat.setChannelCount(1);
    m_audioFormat.setCodec("audio/pcm");
    m_audioFormat.setSampleRate(16000);
    m_audioFormat.setSampleSize(16);
    m_audioFormat.setSampleType(QAudioFormat::SignedInt);

    // 判断设备，查看是否存在
    QAudioDeviceInfo devInfo = QAudioDeviceInfo::defaultInputDevice();
    if(devInfo.isNull())
    {
        qWarning() << "No recording device was found!";
    }
    // 不支持格式，使用最接近格式
    if(!devInfo.isFormatSupported(m_audioFormat))
    { // 当前使用设备是否支持
        qWarning() << "Automatic conversion is made to the nearest format!";
        m_audioFormat = devInfo.nearestFormat(m_audioFormat); // 转换为最接近格式
    }

    m_audioInput = new QAudioInput(devInfo, m_audioFormat, this);
    connect(m_audioInput, &QAudioInput::stateChanged, this, &Recorder::handleStateChanged);

    m_audioInput->setNotifyInterval(10 * 1000);
}

Recorder::~Recorder()
{
    if(m_audioInput != nullptr)
    {
        m_audioInput->stop();
        delete m_audioInput;
    }
}

//单次语音处理
void Recorder::startRecord()
{
    isContinuousSpeechInUse = true;
    if(!initSession())
    {
        emit sigCancelSendVoice();
        return;
    }
    qDebug() << "startRecord";
    m_audioBuffer.open(QIODevice::ReadWrite | QIODevice::Truncate);

    m_audioInput->start(&m_audioBuffer);

    //    QTimer::singleShot(30 * 1000, this, &Recorder::stopRecord);
}

void Recorder::stopRecord()
{
    m_audioInput->stop();
    if(initSession())
    {
        onReadInfo();
    }else
    {
        emit sigCancelSendVoice();
    }
}

void Recorder::cancelRecord()
{
    m_audioInput->stop();
    //emit sigCancelSendVoice();
}

void Recorder::sendAudio(const void *audio_data, unsigned int audio_data_length)
{
    QString error;
    QJsonObject jsonObject;
    jsonObject["format"] = "pcm";
    jsonObject["rate"] = 16000;
    jsonObject["channel"] = 1;
    const char *params = QJsonDocument(jsonObject).toJson(QJsonDocument::Indented);

    SpeechResult ret = speech_recognize_once_async(this->session, params, audio_data, audio_data_length);
    if(ret == SPEECH_SUCCESS)
    {
        KyInfo() << "speech_recognize_once_async successful";
    }
    else
    {
        KyInfo() << "speech_recognize_once_async error:" << ret;
        error = getErrorMsg(ret);
        emit reply(error, true, 0, ret);
        emit sigCancelSendVoice();
        return;
    }
}

void Recorder::onReadInfo()
{
    m_audioBuffer.close();
    m_audioBuffer.open(QIODevice::ReadOnly);
    QByteArray data = m_audioBuffer.readAll();

    qDebug() << "QByteArray data" << data.length();

    if(data.length() != 0)
        sendAudio(data.data(), data.length());
    else
    {
        emit reply("说话时间太短，请重新尝试。", true, 0, 1);
        emit sigCancelSendVoice();
    }
}

bool Recorder::initSession()
{
    QString error;
    //初始化会话时，对创建成功的会话进行判断并进行指针传递，如果创建失败则返回错误码
    if(this->session != nullptr)
        return true;
    void *session = nullptr;
    SpeechResult ret = speech_create_session(&session);
    if(ret == SPEECH_SUCCESS)
    {
        KyInfo() << "speech_create_session successful";
    }
    else
    {
        KyInfo() << "speech_create_session error:" << ret;
        error = getErrorMsg(ret);
        emit reply(error, true, 0, ret);
        emit sigCancelSendVoice();
        return false;
    }

    ret = speech_init_session(session);
    if(ret == SPEECH_SUCCESS)
    {
        KyInfo() << "speech_init_session successful";
    }
    else
    {
        KyInfo() << "speech_init_session error:" << ret;
        error = getErrorMsg(ret);
        emit reply(error, true, 0, ret);
        emit sigCancelSendVoice();
        return false;
    }

    if(!ret)
    {
        this->session = session;
        //单次语音识别使用recognized接收结果
        speech_set_recognized_callback(this->session, speechCallback, this);
    }
    return true;
}

void Recorder::closeSession()
{
    if(this->session != nullptr)
        speech_destroy_session(this->session);
    this->session = nullptr;
    isContinuousSpeechInUse = false;
}

void Recorder::handleStateChanged(QAudio::State state)
{
    qDebug() << "handleStateChanged" << state;
    switch(state)
    {
    case QAudio::StoppedState:
    {
        if(m_audioInput->error() != QAudio::NoError)
        {
            qCritical() << "Sound recording error!";
        }
    }
    break;
    case QAudio::ActiveState:
    {
        qInfo() << "Start recording the sound...";
    }
    break;
    default:
        break;
    }
}
